@@ -33,78 +33,12 @@
 //!
 //!     let position = position(parallax, right_ascension, declination).to_array();
 //!
-//!     println!("The body {} was as the position x: {}, y: {}, z: {} at epoch J2000",
+//!     println!("The body {} was at the position x: {}, y: {}, z: {} at epoch J2000",
 //!     body_name, position[0], position[1], position[2]);
 //! }
 //! ```
 //! The same general principles apply to most functions.
 //!
-//! Now for a more complex example, let's say that we wanted to parse a csv with the collums
-//! `parallax`, `right_ascension`, `declination`, `proper_motion_ra`, `proper_motion_dec` and `radial_velocity`.
-//! Aka not the exact layout found in [`input_data::parse_csv_deserialize`].
-//! We want the position and velocity of the bodies in the list in the cartesian coordinate system printed to the terminal for now.
-//! ```
-//! use spv_rs::position::position;
-//! use spv_rs::velocity::velocity;
-//! use csv::StringRecord;
-//! use serde::Deserialize;
-//! use std::error::Error;
-//!  
-//! #[derive(Debug, Deserialize)]
-//! #[serde(rename_all = "PascalCase")]
-//! struct Collums {
-//!     parallax: f64,
-//!     right_ascension: f64,
-//!     declination: f64,
-//!     proper_motion_ra: f64,
-//!     proper_motion_dec: f64,
-//!     radial_velocity: f64,
-//! }
-//!
-//! fn main() {
-//!     let mut data = vec![];
-//!
-//!     match spv_rs::input_data::parse_csv("some_file.csv") {
-//!         Ok(vec) => data = vec,
-//!         Err(ex) => {
-//!             println!("ERROR -> {}", ex);
-//!         }
-//!     }
-//!  
-//!     let mut deserialized_data = vec![];
-//!
-//!     match deserialize(data) {
-//!         Ok(vec) => deserialized_data = vec,
-//!         Err(ex) => {
-//!             println!("ERROR -> {}", ex);
-//!         }
-//!     }
-//!  
-//!     for i in deserialized_data {
-//!         let position = position(i.parallax, i.right_ascension, i.declination).to_array();
-//!  
-//!         let velocity = velocity(i.parallax, i.right_ascension, i.declination,
-//!             i.proper_motion_ra, i.proper_motion_dec, i.radial_velocity).to_array();
-//!  
-//!         println!("This bodies position is: ({}, {}, {}) and it's velocity is ({}, {}, {})",
-//!             position[0], position[1], position[2], velocity[0], velocity[1], velocity[2])
-//!     }
-//! }
-//!
-//! fn deserialize(
-//!     data: std::vec::Vec<StringRecord>
-//! ) -> Result<std::vec::Vec<Collums>, Box<dyn Error>> {
-//!     let mut vec = vec![];
-//!
-//!     for result in data {
-//!         let record: Collums = result.deserialize(None)?;
-//!         vec.push(record);
-//!     }
-//!  
-//!     Ok(vec)
-//! }
-//! ```
-//!
 //! ### Extra
 //!
 //! Feel free to propose additions/changes, file issues and or help with the project over on [GitHub](https://github.com/AlbinSjoegren/SPV)!
@@ -115,8 +49,8 @@ pub mod position {
     use super::common::semi_parameter;
     use super::common::true_anomaly;
     use super::coordinate_transforms::euler_angle_transformations;
-    use glam::f64::{DVec2, DVec3};
     use glam::f32::Vec3;
+    use glam::f64::{DVec2, DVec3};
 
     /// Position of a single celestial object relative to the sun.
     /// Can be used in conjuction with companion functions to place a twobody system relative to the sun.
@@ -316,7 +250,7 @@ pub mod velocity {
     /// due to the fact that everything is on a plane in 2D.
     pub fn companion_velocity(a: f64, e: f64, period: f64, t_p: f64) -> DVec2 {
         //Prep Values
-        let mu = standard_gravitational_parameter(a, e);
+        let mu = standard_gravitational_parameter(a, period);
         let p = semi_parameter(a, e);
         let v = true_anomaly(e, period, t_p);
 
@@ -341,7 +275,7 @@ pub mod velocity {
         i: f64,
     ) -> DVec3 {
         //Prep Values
-        let mu = standard_gravitational_parameter(a, e);
+        let mu = standard_gravitational_parameter(a, period);
         let p = semi_parameter(a, e);
         let v = true_anomaly(e, period, t_p);
 
@@ -373,7 +307,7 @@ pub mod velocity {
 
     /// Just the companion velocity but as a value and not coordinates.
     pub fn companion_velocity_value(a: f64, e: f64, period: f64, t_p: f64) -> f64 {
-        let mu = standard_gravitational_parameter(a, e);
+        let mu = standard_gravitational_parameter(a, period);
         let epsilon = specific_mechanical_energy(a, e);
         let r = radius(a, e, period, t_p);
 
@@ -384,7 +318,7 @@ pub mod velocity {
 /// Set of common functions used by `spv-rs` exposed if you want to used them for your own calculations.
 pub mod common {
     use super::coordinate_transforms::euler_angle_transformations;
-    use super::position::companion_relative_position; 
+    use super::position::companion_relative_position;
     use super::velocity::companion_relative_velocity;
     use glam::f64::DVec3;
 
@@ -531,9 +465,9 @@ pub mod common {
     }
 
     /// Calculates the specific angular momentum value
-    pub fn specific_angular_momentum_value(a: f64, e: f64) -> f64 {
+    pub fn specific_angular_momentum_value(a: f64, e: f64, period: f64) -> f64 {
         let p = semi_parameter(a, e);
-        let mu = standard_gravitational_parameter(a, e);
+        let mu = standard_gravitational_parameter(a, period);
 
         (mu * p).sqrt()
     }
@@ -555,25 +489,24 @@ pub mod common {
     }
 
     /// Calculates the stadard gravitational parameter
-    pub fn standard_gravitational_parameter(a: f64, e: f64) -> f64 {
+    pub fn standard_gravitational_parameter(a: f64, period: f64) -> f64 {
+        let period_si = period * 31557600.;
         let a_si = au_to_m(a);
-        let p_si = semi_parameter(a, e);
 
-        ((a_si.powf(3.)) * 4. * (std::f64::consts::PI.powf(2.))) / (p_si.powf(2.))
+        ((a_si.powf(3.)) * 4. * (std::f64::consts::PI.powf(2.))) / (period_si.powf(2.))
     }
 
     /// Specific mechanical energy (used by other equation but exposed here if you need it)
-    pub fn specific_mechanical_energy(a: f64, e: f64) -> f64 {
+    pub fn specific_mechanical_energy(a: f64, period: f64) -> f64 {
         let a_si = au_to_m(a);
-        let mu = standard_gravitational_parameter(a, e);
+        let mu = standard_gravitational_parameter(a, period);
 
         0. - (mu / (2. * a_si))
     }
 
     /// If you dind't have the period already
-    pub fn period(a: f64, e: f64) -> f64 {
+    pub fn period(a: f64, mu: f64) -> f64 {
         let a_si = au_to_m(a);
-        let mu = standard_gravitational_parameter(a, e);
 
         2. * std::f64::consts::PI * (((a_si.powf(3.)) / mu).sqrt())
     }
@@ -587,9 +520,9 @@ pub mod common {
     }
 
     /// Mean motion or n
-    pub fn mean_motion(a: f64, e: f64) -> f64 {
+    pub fn mean_motion(a: f64, period: f64) -> f64 {
         let a_si = au_to_m(a);
-        let mu = standard_gravitational_parameter(a, e);
+        let mu = standard_gravitational_parameter(a, period);
 
         (mu / (a_si.powf(3.))).sqrt()
     }
@@ -627,6 +560,16 @@ pub mod common {
     pub fn au_to_m(a: f64) -> f64 {
         a * 1000. * 149597870.7
     }
+
+    ///
+    pub fn apparent_mag_to_absolute_mag(parallax: f64, apparent_magnitude: f64) -> f64 {
+        apparent_magnitude + 5. * (parallax.log10() + 1.)
+    }
+
+    ///
+    pub fn temperature(b_v_index: f64) -> f64 {
+        4600. * ((1. / ((0.92 * b_v_index) + 1.7)) + (1. / ((0.92 * b_v_index) + 0.62)))
+    }
 }
 
 /// Transform fucntions used by `spv-rs` but exposed her if you want to use them yourself.
@@ -670,65 +613,189 @@ pub mod coordinate_transforms {
 /// To get a csv if you got some other format from something like [Vizier](https://vizier.cds.unistra.fr/viz-bin/VizieR)
 /// I would recomend a tool like [Topcat](http://www.star.bris.ac.uk/~mbt/topcat/).
 pub mod input_data {
-    use csv::{ReaderBuilder, StringRecord, Terminator};
-    use serde::Deserialize;
+    use csv::{ReaderBuilder, Terminator};
     use std::error::Error;
 
-    /// General usecase parsing function for csv files, no specific structure required.
-    pub fn parse_csv(filename: &str) -> Result<std::vec::Vec<StringRecord>, Box<dyn Error>> {
+    /// General usecase parsing function for csv files.
+    pub fn parse_csv<T: for<'de> serde::Deserialize<'de>>(
+        filename: &str,
+        has_headers: bool,
+        cols_split: u8,
+        row_split: u8,
+    ) -> Result<std::vec::Vec<T>, Box<dyn Error>> {
         let mut vec = vec![];
         let mut rdr = ReaderBuilder::new()
-            .delimiter(b',')
-            .terminator(Terminator::Any(b'\n'))
-            .has_headers(false)
-            .from_path(filename)?;
-        for result in rdr.records().flatten() {
-            let record = result;
-            vec.push(record);
-        }
-        Ok(vec)
-    }
-
-    /// Struct defining a set of collumn variables and types for cav parsing with specific input structure.
-    #[derive(Debug, Deserialize)]
-    #[serde(rename_all = "PascalCase")]
-    pub struct Collums {
-        /// Right ascension in epoch J2000.
-        pub ra_j2000: f32,
-        /// Declination in epoch J2000.
-        pub dec_j2000: f32,
-        /// Hipparcos catalogue number for the body.
-        pub hip: u32,
-        /// Common name for the body.
-        pub name: String,
-        /// Right ascension part of the proper motion in epoch J2000.
-        pub pm_ra_j2000: f32,
-        /// Declination part of the proper motion in epoch J2000.
-        pub pm_dec_j2000: f32,
-        /// Parallax in epoch J2000
-        pub plx_j2000: f32,
-        /// Radial velocity in epoch J2000
-        pub rv_j2000: f32,
-        /// Visual magnitude in epoch J2000
-        pub vmag_j2000: f32,
-    }
-
-    /// csv parsing with deserializsation using specific collum layout found in [Collums]
-    pub fn parse_csv_deserialize(filename: &str) -> Result<std::vec::Vec<Collums>, Box<dyn Error>> {
-        let mut vec = vec![];
-        let mut rdr = ReaderBuilder::new()
-            .delimiter(b',')
-            .terminator(Terminator::Any(b'\n'))
-            .has_headers(false)
+            .delimiter(cols_split)
+            .terminator(Terminator::Any(row_split))
+            .has_headers(has_headers)
             .from_path(filename)?;
         for result in rdr.deserialize() {
-            let record: Collums = result?;
+            let record: T = result?;
             vec.push(record);
         }
         Ok(vec)
     }
 }
 
+/// Basic csv writing utility for saving large datasets to file.
+pub mod output_data {
+    use csv::{Terminator, WriterBuilder};
+    use std::error::Error;
+
+    /// General usecase writing function for csv files.
+    pub fn write_csv<T: serde::Serialize>(
+        output_filename: &str,
+        has_headers: bool,
+        cols_split: u8,
+        row_split: u8,
+        vec: std::vec::Vec<T>,
+    ) -> Result<(), Box<dyn Error>> {
+        let mut writer = WriterBuilder::new()
+            .delimiter(cols_split)
+            .terminator(Terminator::Any(row_split))
+            .has_headers(has_headers)
+            .from_path(output_filename)?;
+
+        for n in vec {
+            writer.serialize(n)?;
+        }
+
+        Ok(())
+    }
+}
+
+/// Specific usecase functions for NBSS
+pub mod nbss {
+    use serde::Deserialize;
+    use serde::Serialize;
+
+    /// NBSS Input struct
+    #[derive(Debug, Deserialize)]
+    #[serde(rename_all = "PascalCase")]
+    pub struct NBSSInputCollums {
+        pub name: String,
+        pub mass: f64,
+        pub radius: f64,
+        pub reference_body: String,
+        pub a: f64,
+        pub e: f64,
+        pub period: f64,
+        pub time_since_periapsis: f64,
+        pub lotn: f64,
+        pub aop: f64,
+        pub i: f64,
+    }
+
+    /// NBSS Output struct
+    #[derive(Serialize)]
+    #[serde(rename_all = "PascalCase")]
+    pub struct NBSSOutputCollums {
+        pub name: String,
+        pub mass: f64,
+        pub radius: f64,
+        pub reference_body: String,
+        pub e: f64,
+        pub semi_parameter: f64,
+        pub position_x: f64,
+        pub position_y: f64,
+        pub position_z: f64,
+        pub velocity_x: f64,
+        pub velocity_y: f64,
+        pub velocity_z: f64,
+    }
+
+    /// NBSS Calc struct (can be used as an example of what you can do with SPV)
+    pub fn position_and_velocity_twobody_serialized(input_filename: &str, output_filename: &str) {
+        let mut file: std::vec::Vec<NBSSInputCollums> = vec![];
+        match super::input_data::parse_csv(input_filename, false, b',', b'\n') {
+            Ok(vec) => file = vec,
+            Err(ex) => {
+                println!("ERROR -> {}", ex);
+            }
+        };
+
+        let mut vec = vec![];
+
+        for n in file {
+            if n.a == 0. && n.e == 0. {
+                let row = NBSSOutputCollums {
+                    name: n.name,
+                    mass: n.mass,
+                    radius: n.radius,
+                    reference_body: n.reference_body,
+                    e: 0.,
+                    semi_parameter: 0.,
+                    position_x: 0.,
+                    position_y: 0.,
+                    position_z: 0.,
+                    velocity_x: 0.,
+                    velocity_y: 0.,
+                    velocity_z: 0.,
+                };
+
+                vec.push(row);
+            } else {
+                let pos = super::position::companion_relative_position(
+                    n.a,
+                    n.e,
+                    n.period,
+                    n.time_since_periapsis,
+                    n.lotn,
+                    n.aop,
+                    n.i,
+                )
+                .to_array();
+                let vel = super::velocity::companion_relative_velocity(
+                    n.a,
+                    n.e,
+                    n.period,
+                    n.time_since_periapsis,
+                    n.lotn,
+                    n.aop,
+                    n.i,
+                )
+                .to_array();
+                let semi_parameter = super::common::semi_parameter(n.a, n.e);
+
+                let row = NBSSOutputCollums {
+                    name: n.name,
+                    mass: n.mass,
+                    radius: n.radius,
+                    reference_body: n.reference_body,
+                    e: n.e,
+                    semi_parameter,
+                    position_x: pos[0],
+                    position_y: pos[1],
+                    position_z: pos[2],
+                    velocity_x: vel[0],
+                    velocity_y: vel[1],
+                    velocity_z: vel[2],
+                };
+
+                vec.push(row);
+            }
+        }
+
+        match super::output_data::write_csv(output_filename, false, b',', b'\n', vec) {
+            Ok(_) => (),
+            Err(ex) => {
+                println!("ERROR -> {}", ex);
+            }
+        };
+    }
+
+    //WIP
+    /*
+    pub fn position_and_velocity_relative_twobody(input_filename: &str, output_filename: &str) {
+
+    }
+
+    pub fn position_and_velocity_relative(input_filename: &str, output_filename: &str) {
+
+    }
+    */
+}
+
 // WIP
 /*
 pub fn above(
